@@ -4,8 +4,9 @@ source("utils.R")
 
 test_that("Python integers can be compared", {
   skip_if_no_python()
-  a <- py$int(1)
-  b <- py$int(2)
+  builtins <- import_builtins(convert = FALSE)
+  a <- builtins$int(1)
+  b <- builtins$int(2)
   expect_true(a < b)
   expect_true(b > a)
   expect_true(a == a)
@@ -16,8 +17,9 @@ test_that("Python integers can be compared", {
 
 test_that("Python objects with the same reference are equal", {
   skip_if_no_python()
-  list_fn <- py$list
-  expect_true(list_fn == py$list)
+  builtins <- import_builtins(convert = FALSE)
+  list_fn <- builtins$list
+  expect_true(list_fn == builtins$list)
 })
 
 
